/*
 * cf533_ctl.c
 *
 *  Created on: Nov 11, 2011
 *      Author: lsantos@pd3.com.br
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <string.h>

#include "CFontz633io.h"
#include "configuration.h"

//#define USB

#ifdef USB
#define CF533DEVICE	"/dev/ttyUSB0"
int is_usb = 1;
#else
#define CF533DEVICE	"/dev/ttyS0"
int is_usb = 0;
#endif

#define CF533WIDTH	16

int fd;

struct termios old_portset;

extern ReceiveBuffer receivebuffer;

#if 1
/* from CFontzPacket.c : CFontzPacket_init */
int Serial_Init(char *devname, int baud_rate)
{
	struct termios portset;

	fd = open(CF533DEVICE, (is_usb) ? (O_RDWR | O_NOCTTY) : (O_RDWR | O_NOCTTY | O_NDELAY));
	if (fd == -1) {
		return -1;
	}

	tcgetattr(fd, &portset);
	tcgetattr(fd, &old_portset);

	/* We use RAW mode */
#ifdef HAVE_CFMAKERAW
	/* The easy way */
	cfmakeraw(&portset);
#else
	/* The hard way */
	portset.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
	portset.c_oflag &= ~OPOST;
	portset.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
	portset.c_cflag &= ~(CSIZE | PARENB | CRTSCTS);
	portset.c_cflag |= CS8 | CREAD | CLOCAL;
#endif
	if (is_usb) {
		portset.c_cc[VMIN] = 0;
		portset.c_cc[VTIME] = 0;
	}

	/* Set port speed */
	cfsetospeed(&portset, B19200);
	cfsetispeed(&portset, B0);

	/* Do it... */
	tcsetattr(fd, TCSANOW, &portset);

	return 0;
}
#else
int Serial_Init(char *devname, int baud_rate)
{
	int
	brate;
	struct
	termios term;

	//open device
	fd = open(devname, O_RDWR | O_NOCTTY | O_NONBLOCK);

	if(fd <= 0)
	{
		printf("Serial_Init:: Open() failed\n");
		return(1);
	}

	//get baud rate constant from numeric value
	switch (baud_rate)
	{
		case 19200:
		brate=B19200;
		break;
		case 115200:
		brate=B115200;
		break;
		default:
		printf("Serial_Init:: Invalid baud rate: %d (must be 19200 or 115200)\n", baud_rate);
		return(2);
	}
	//get device struct
	if(tcgetattr(fd, &term) != 0)
	{
		printf("Serial_Init:: tcgetattr() failed\n");
		return(3);
	}

	//input modes
	term.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|INPCK|ISTRIP|INLCR|IGNCR|ICRNL
			|IXON|IXOFF);
	term.c_iflag |= IGNPAR;

	//output modes
	term.c_oflag &= ~(OPOST|ONLCR|OCRNL|ONOCR|ONLRET|OFILL
			|OFDEL|NLDLY|CRDLY|TABDLY|BSDLY|VTDLY|FFDLY);

	//control modes
	term.c_cflag &= ~(CSIZE|PARENB|PARODD|HUPCL|CRTSCTS);
	term.c_cflag |= CREAD|CS8|CSTOPB|CLOCAL;

	//local modes
	term.c_lflag &= ~(ISIG|ICANON|IEXTEN|ECHO);
	term.c_lflag |= NOFLSH;

	//set baud rate
	cfsetospeed(&term, brate);
	cfsetispeed(&term, brate);

	//set new device settings
	if(tcsetattr(fd, TCSANOW, &term) != 0)
	{
		printf("Serial_Init:: tcsetattr() failed\n");
		return(4);
	}

	printf("Serial_Init:: success\n");
	return(0);
}
#endif
int close_serial(void)
{
	tcsetattr(fd, TCSANOW, &old_portset);
	close(fd);
	return 0;
}

int lcd_hw_clear(void)
{
	send_zerobyte_message(fd, CF633_Clear_LCD_Screen);

	return 0;
}

int lcd_reboot(void)
{
	unsigned char data[8];

	memset(data, '\0', sizeof(data));
	data[0] = 8;
	data[1] = 18;
	data[2] = 99;
	send_bytes_message(fd, CF633_Reboot, 3, data);

	return 0;
}

int lcd_print_strings(char *first_line, char *second_line)
{
	unsigned char data[64];
	int len;
	/* send first line */
	if (first_line != NULL && strlen(first_line) > 0) {
		len = strlen(first_line);
		if (len > CF533WIDTH)
			len = CF533WIDTH;
		memset(data, '\0', sizeof(data));
		memcpy(data + 2, first_line, len);
		send_bytes_message(fd, CF633_Send_Data_to_LCD, len + 2, data);
	}
	if (second_line != NULL && strlen(second_line) > 0) {
		len = strlen(second_line);
		if (len > CF533WIDTH)
			len = CF533WIDTH;
		memset(data, '\0', sizeof(data));
		memcpy(data + 2, second_line, len);
		data[1] = 1; /* second row */
		send_bytes_message(fd, CF633_Send_Data_to_LCD, len + 2, data);
	}
	return 0;
}

int lcd_set_bootstate(void)
{
	send_zerobyte_message(fd, CF633_Store_Current_State_As_Boot_State);

	return 0;
}

#define AUTO_POLARITY		0x01	//Automatically detects polarity for reset and
//power (recommended)
#define RESET_INVERT		0x02	//Reset pin drives high instead of low
#define POWER_INVERT		0x04	//Power pin drives high instead of low
#define LCD_OFF_IF_HOST_IS_OFF	0x10
#define KEYPAD_RESET		0x20
#define KEYPAD_POWER_ON		0x40
#define KEYPAD_POWER_OFF	0x80

int lcd_set_atx(unsigned char mask)
{
	unsigned char data[8];

	if (mask != 0) {
		memset(data, '\0', sizeof(data));
		data[0] = 1;
		data[1] = 0;
		data[2] = 3;
		send_bytes_message(fd, CF633_Set_GPIO_Pin, 3, data);

		memset(data, '\0', sizeof(data));
		data[0] = 2;
		data[1] = 0;
		data[2] = 2;
		send_bytes_message(fd, CF633_Set_GPIO_Pin, 3, data);

		memset(data, '\0', sizeof(data));
		data[0] = 3;
		data[1] = 0;
		data[2] = 2;
		send_bytes_message(fd, CF633_Set_GPIO_Pin, 3, data);

		memset(data, '\0', sizeof(data));
		data[0] = mask;
		data[1] = 16;	// One half second power pulses
#ifdef USB
		                data[2] = 1;
#else
		data[2] = 0;
#endif
		send_bytes_message(fd, CF633_Set_ATX_Power_Switch_Functionality, 3, data);
	} else {
		send_onebyte_message(fd, CF633_Set_ATX_Power_Switch_Functionality, 0);
	}

	return 0;
}

int lcd_set_contrast(unsigned char c)
{
	send_onebyte_message(fd, CF633_Set_LCD_Contrast, c);

	return 0;
}

int lcd_set_backlight(unsigned char lcd, unsigned char keys)
{
	unsigned char data[8];

	memset(data, '\0', sizeof(data));
	data[0] = (lcd > 100 ? 100 : lcd);
	data[1] = (keys > 100 ? 100 : keys);
	send_bytes_message(fd, CF633_Set_LCD_And_Keypad_Backlight, 2, data);

	return 0;
}

int main(void)
{
	struct config *conf;
	char *name;
	int ret;

	if (Serial_Init(CF533DEVICE, 19200))
		return -1;

	config_init(&conf);
	ret = config_read_conf(conf, "/etc/AG800/config");

	if (ret!=0){
		name = "NULL";
	} else {
		name = config_get_value(conf, "Screen_home");
	}


	lcd_reboot();

	sleep(1);

	lcd_set_contrast(15);

	lcd_set_backlight(50, 50);

	lcd_hw_clear();


        lcd_print_strings(name, \
                          " Iniciando ... ");
	sleep(1);

        lcd_print_strings(name, \
                          " Iniciando ... ");

	config_destroy(&conf);

	lcd_set_atx(
	                KEYPAD_POWER_OFF | KEYPAD_POWER_ON | KEYPAD_RESET | LCD_OFF_IF_HOST_IS_OFF
	                                | AUTO_POLARITY);

	lcd_set_bootstate();

	sleep(1);

	lcd_reboot();

	close_serial();

	return 0;
}
