/*
 * configuration.h
 *
 *  Created on: May 23, 2011
 *      Author: Rodrigo Ferreira <rferreira@pd3.com.br>
 */

#ifndef CONFIGURATION_H_
#define CONFIGURATION_H_

#include "htable.h"

struct config {
	int initialized;
	int errors;
	htable *config_table;
};

void config_destroy(struct config **conf);
void config_init(struct config **conf);
int config_read_conf(struct config *conf, const char *file);
char *config_get_value(struct config *conf, const char *key);
int config_get_errors(struct config *conf);
int config_save_conf(struct config *conf, const char* filepath);
int config_read_defaults(struct config *conf, const char *file);

/**
 * (Re)Inserts in the hash table of the old_conf the same old VALUES found in it
 * but using the new configuration KEYS.
 *
 * @param old_conf pointer to the configuration to be updated
 * @param old_version_match the version string of the old_config (don't updates if it diverges)
 * @param filepath path to the file with the new configuration
 * @param new_version_match the version string of the new_config (don't updates if it diverges)
 * @param version_key the configuration key which specifies the value used to compare versions
 * @param replace_array array of strings with the old keys to be replaced and the new replacing keys, side by side
 * @param replace_count number of keys to be replaced (half the size of the replace_array)
 */
int configuration_update_keys(struct config *old_conf,
                              char *old_version_match,
                              const char* filepath,
                              char *new_version_match,
                              char *version_key,
                              char **replace_array,
                              int replace_count);

#endif /* CONFIGURATION_H_ */
