#ifndef HTABLE_H_
#define HTABLE_H_

/**
 * @defgroup HashTable Hash Table implementation
 * @ingroup DataStructures
 * @{
 */

/**
 * Hash table object definition.
 */
typedef struct _htable htable;

/**
 * Internal hash table node.
 */
struct _htable_node {
  /**> next node */
  struct _htable_node *next;
  /**> calculated hash for the node */
  unsigned long int hash;
  /**> key */
  char *key;
  /**> value */
  void *data;
};

/**
 * Hash table internal structure.
 */
struct _htable {
  /**> maximum number of elements */
  int size;
  /**> number of allocated elements */
  int num_elem;
  /**> high density factor, default is 70% full */
  int high_fill_factor;
  /**> low density factor, default is 30% full */
  int low_fill_factor;
  /**> user destroy data function */
  void (*destroy_data)(void *);
  /**> the table containing the nodes */
  struct _htable_node **table;
};

htable *htable_create(int size, void (*destroy_data)(void *));
void htable_destroy(htable *ht);

int htable_insert(htable *ht, char *key, void *v);
void htable_remove(htable *ht, char *key);
void htable_remove_entry(htable *ht, char *key);

void *htable_lookup(htable *ht, char *key);
int htable_update(htable *ht, char *key, void *d);

void *htable_walk(htable *ht, char *key);
void htable_print(htable *ht);

int htable_num_elements(htable *ht);

void htable_set_resize_high_density(htable *ht, int fill_factor);
void htable_set_resize_low_density(htable *ht, int fill_factor);

/** @} */

#endif /* HTABLE_H_ */
